@@ -2,7 +2,7 @@
 """
     Policy Sentry is a tool for generating least-privilege IAM Policies.
 """
-__version__ = "0.8.0.5"
+__version__ = "0.8.0.7"
 import click
 from policy_sentry import command
 
@@ -15,7 +15,7 @@ def policy_sentry():
     """
 
 
-policy_sentry.add_command(command.initialize.initialize)
+policy_sentry.add_command(command.initialize.initialize_command)
 policy_sentry.add_command(command.write_policy.write_policy)
 policy_sentry.add_command(command.create_template.create_template)
 policy_sentry.add_command(command.query.query)
