@@ -1,8 +1,9 @@
 """
-Methods that execute specific queries against the SQLite database for the ACTIONS table.
+Methods that execute specific queries against the database for the ACTIONS table.
 This supports the Policy Sentry query functionality
 """
 import logging
+import functools
 from policy_sentry.shared.iam_data import iam_definition, get_service_prefix_data
 from policy_sentry.querying.all import get_all_service_prefixes
 from policy_sentry.querying.arns import get_matching_raw_arn, get_resource_type_name_with_raw_arn
@@ -12,12 +13,15 @@ all_service_prefixes = get_all_service_prefixes()
 logger = logging.getLogger(__name__)
 
 
+@functools.lru_cache(maxsize=1024)
 def get_actions_for_service(service_prefix):
     """
     Get a list of available actions per AWS service
 
-    :param service_prefix: An AWS service prefix, like `s3` or `kms`
-    :return: A list of actions
+    Arguments:
+        service_prefix: List: An AWS service prefix, like `s3` or `kms`
+    Returns:
+        A list of actions
     """
     service_prefix_data = get_service_prefix_data(service_prefix)
     results = []
@@ -26,13 +30,17 @@ def get_actions_for_service(service_prefix):
     return results
 
 
+@functools.lru_cache(maxsize=1024)
 def get_action_data(service, action_name):
     """
     Get details about an IAM Action in JSON format.
 
-    :param service: An AWS service prefix, like `s3` or `kms`. Case insensitive.
-    :param action_name: The name of an AWS IAM action, like `GetObject`. To get data about all actions in a service, specify "*". Case insensitive.
-    :return: A dictionary containing metadata about an IAM Action.
+    Arguments:
+        service: An AWS service prefix, like `s3` or `kms`. Case insensitive.
+        action_name: The name of an AWS IAM action, like `GetObject`. To get data about all actions in a service, specify "*". Case insensitive.
+
+    Returns:
+        List: A dictionary containing metadata about an IAM Action.
     """
     results = []
     action_data_results = {}
@@ -89,8 +97,11 @@ def get_actions_that_support_wildcard_arns_only(service_prefix):
     Get a list of actions that do not support restricting the action to resource ARNs.
     Set service to "all" to get a list of actions across all services.
 
-    :param service_prefix: A single AWS service prefix, like `s3` or `kms`
-    :return: A list of actions
+    Arguments:
+        service_prefix: A single AWS service prefix, like `s3` or `kms`
+
+    Returns:
+        List: A list of actions that do not support resource ARN constraints
     """
     results = []
     if service_prefix == "all":
@@ -117,9 +128,11 @@ def get_actions_at_access_level_that_support_wildcard_arns_only(
     Get a list of actions at an access level that do not support restricting the action to resource ARNs.
     Set service to "all" to get a list of actions across all services.
 
-    :param service_prefix: A single AWS service prefix, like `s3` or `kms`
-    :param access_level: An access level as it is written in the database, such as 'Read', 'Write', 'List', 'Permisssions management', or 'Tagging'
-    :return: A list of actions
+    Arguments:
+        service_prefix: A single AWS service prefix, like `s3` or `kms`
+        access_level: An access level as it is written in the database, such as 'Read', 'Write', 'List', 'Permisssions management', or 'Tagging'
+    Returns:
+        List: A list of actions at that access level that do not support resource ARN constraints
     """
     results = []
     if service_prefix == "all":
@@ -148,9 +161,12 @@ def get_actions_with_access_level(service_prefix, access_level):
     """
     Get a list of actions in a service under different access levels.
 
-    :param service_prefix: A single AWS service prefix, like `s3` or `kms`
-    :param access_level: An access level as it is written in the database, such as 'Read', 'Write', 'List', 'Permisssions management', or 'Tagging'
-    :return: A list of actions
+    Arguments:
+        service_prefix: A single AWS service prefix, like `s3` or `kms`
+        access_level: An access level as it is written in the database, such as 'Read', 'Write', 'List', 'Permisssions management', or 'Tagging'
+
+    Returns:
+        List: A list of actions with that access level and service prefix
     """
     results = []
     if service_prefix == "all":
@@ -173,10 +189,12 @@ def get_actions_with_arn_type_and_access_level(
     """
     Get a list of actions in a service under different access levels, specific to an ARN format.
 
-    :param service_prefix: A single AWS service prefix, like `s3` or `kms`
-    :param resource_type_name: The ARN type name, like `bucket` or `key`
-    :param access_level: Access level like "Read" or "List" or "Permissions management"
-    :return: A list of actions
+    Arguments:
+        service_prefix: A single AWS service prefix, like `s3` or `kms`
+        resource_type_name: The ARN type name, like `bucket` or `key`
+        access_level: Access level like "Read" or "List" or "Permissions management"
+    Return:
+        List: A list of actions that have that ARN type and Access level
     """
     service_prefix_data = get_service_prefix_data(service_prefix)
     results = []
@@ -192,7 +210,14 @@ def get_actions_with_arn_type_and_access_level(
 
 
 def get_actions_matching_arn(arn):
-    """Given a user-supplied arn, get a list of all actions that can match it."""
+    """
+    Given a user-supplied ARN, get a list of all actions that correspond to that ARN.
+
+    Arguments:
+        arn: A user-supplied arn
+    Returns:
+        List: A list of all actions that can match it.
+    """
     raw_arn = get_matching_raw_arn(arn)
     resource_type_name = get_resource_type_name_with_raw_arn(raw_arn)
     service_prefix = get_service_from_arn(raw_arn)
@@ -212,9 +237,11 @@ def get_actions_matching_condition_key(service_prefix, condition_key):
     """
     Get a list of actions under a service that allow the use of a specified condition key
 
-    :param service_prefix: A single AWS service prefix
-    :param condition_key: The condition key to look for.
-    :return: A list of actions
+    Arguments:
+        service_prefix: A single AWS service prefix
+        condition_key: The condition key to look for.
+    Returns:
+        List: A list of actions
     """
     results = []
     if service_prefix == "all":
@@ -239,9 +266,9 @@ def get_actions_matching_condition_key(service_prefix, condition_key):
 #     """
 #     Get a list of IAM Actions matching a condition key, CRUD level, and raw ARN format.
 #
-#     :param condition_key: A condition key, like aws:TagKeys
-#     :param access_level: Access level that matches the database value. "Read", "Write", "List", "Tagging", or "Permissions management"
-#     :param raw_arn: The raw ARN format in the database, like arn:${Partition}:s3:::${BucketName}
+#         condition_key: A condition key, like aws:TagKeys
+#         access_level: Access level that matches the database value. "Read", "Write", "List", "Tagging", or "Permissions management"
+#         raw_arn: The raw ARN format in the database, like arn:${Partition}:s3:::${BucketName}
 #     :return: List of IAM Actions
 #     """
 #     print()
@@ -253,9 +280,11 @@ def remove_actions_not_matching_access_level(actions_list, access_level):
     """
     Given a list of actions, return a list of actions that match an access level
 
-    :param actions_list: A list of actions
-    :param access_level: 'read', 'write', 'list', 'tagging', or 'permissions-management'
-    :return: Updated list of actions, where the actions not matching the requested access level are removed.
+    Arguments:
+        actions_list: A list of actions
+        access_level: 'read', 'write', 'list', 'tagging', or 'permissions-management'
+    Returns:
+        List: An Updated list of actions, where the actions not matching the requested access level are removed.
     """
     new_actions_list = []
 
@@ -266,8 +295,8 @@ def remove_actions_not_matching_access_level(actions_list, access_level):
             if "privileges" in service_prefix_data:
                 for action_instance in service_prefix_data["privileges"]:
                     if action_instance.get("access_level") == access_level:
-                        logger.debug(f"remove_actions_not_matching_access_level: Provided access level is {access_level}, "
-                                     f"matches {action_instance.get('access_level')}")
+                        # logger.debug(f"remove_actions_not_matching_access_level: Provided access level is {access_level}, "
+                        #              f"matches {action_instance.get('access_level')}")
                         if action_instance.get("privilege").lower() == some_action.lower():
                             this_result = f"{some_service_prefix}:{action_instance.get('privilege')}"
                             break
@@ -301,13 +330,15 @@ def get_dependent_actions(actions_list):
     fifth column of the Resources, Actions, and Condition keys tables. If it does, add the dependent actions
     to the list, and return the updated list.
 
-    It includes the original action in there as well. So, if you supply kms:CreateCustomKeyStore, it will give you kms:CreateCustomKeyStore as well as cloudhsm:DescribeClusters
+    It includes the original action in there as well. So, if you supply `kms:CreateCustomKeyStore`, it will give you `kms:CreateCustomKeyStore` as well as `cloudhsm:DescribeClusters`
 
     To get dependent actions for a single given IAM action, just provide the action as a list with one item, like this:
-    get_dependent_actions(db_session, ['kms:CreateCustomKeystore'])
+    `get_dependent_actions(db_session, ['kms:CreateCustomKeystore'])`
 
-    :param actions_list: A list of actions to use in querying the database for dependent actions
-    :return: Updated list of actions, including dependent actions if applicable.
+    Arguments:
+        actions_list: A list of actions to use in querying the database for dependent actions
+    Returns:
+        List: Updated list of actions, including dependent actions if applicable.
     """
     new_actions_list = []
     for action in actions_list:
@@ -328,9 +359,10 @@ def remove_actions_that_are_not_wildcard_arn_only(actions_list):
     """
     Given a list of actions, remove the ones that CAN be restricted to ARNs, leaving only the ones that cannot.
 
-    :param actions_list: A list of actions
-    :return: An updated list of actions
-    :rtype: list
+    Arguments:
+        actions_list: A list of actions
+    Returns:
+        List: An updated list of actions
     """
     # remove duplicates, if there are any
     actions_list_unique = list(dict.fromkeys(actions_list))
@@ -347,9 +379,14 @@ def remove_actions_that_are_not_wildcard_arn_only(actions_list):
 
 def get_privilege_info(service, action):
     """
-    Given a service, like "s3"
-    and an action, like "ListBucket"
-    return the info from the docs about that action, along with some of the info from the docs
+    Given a service, like `s3` and an action name, like `ListBucket`, return info about that action.
+
+    Arguments:
+        service: The service prefix, like `s3`
+        action: An action name, like `ListBucket`
+
+    Returns:
+        List: The info from the docs about that action, along with some of the info from the docs
     """
     for service_info in iam_definition:
         if service_info["prefix"] == service:
