@@ -2,6 +2,7 @@
 import os
 import json
 import logging
+import functools
 
 # On initialization, load the IAM data
 iam_definition_path = os.path.join(
@@ -11,12 +12,15 @@ iam_definition = json.load(open(iam_definition_path, "r"))
 logger = logging.getLogger(__name__)
 
 
+@functools.lru_cache(maxsize=1024)
 def get_service_prefix_data(service_prefix):
     """
     Given an AWS service prefix, return a large dictionary of IAM privilege data for processing and analysis.
 
-    :param service_prefix: An AWS service prefix, like s3 or ssm
-    :return:
+    Arguments:
+        service_prefix: An AWS service prefix, like s3 or ssm
+    Returns:
+        A list of metadata about that service
     """
     result = list(filter(lambda item: item["prefix"] == service_prefix, iam_definition))
     try:
